<?php
/**
* some string handling functions
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_strings.php,v 1.29 2006/05/24 14:50:55 weinert Exp $
*/


/**
* some string handling functions
*
* @package papaya
*/
class papaya_strings {

  /**
  * Escapes entities and some chars to be uses in the xml-output.
  *
  * @param string $str
  * @param boolean $escapeTags Escape < and >, default value FALSE
  * @access public
  * @return string
  */
  function entityToXML($str, $escapeTags = FALSE) {
    $result = $str;
    $result = preg_replace('/\&((amp)|(quot)|([gl]t)|(#((\d+)|(x[a-fA-F\d]{2})|(x[a-fA-F\d]{4}))))\;/i','#||\\1||#',$result);
    $result = strtr($result, array_flip(get_html_translation_table(HTML_ENTITIES)));
    $result = str_replace('&','&amp;',$result);
    $result = preg_replace('/\#\|\|([a-z\d\#]+)\|\|\#/i','&\\1;',$result);
    $result = str_replace('&amp;amp;','&amp;', $result);
    if ($escapeTags) {
      $result = str_replace(array('<', '>'), array('&lt;', '&gt;'), $result);
    }
    return $result;
  }
  
  /**
  * Escape HTML special chars
  *
  * @param $str
  * @access public
  * @return string
  */
  function escapeHTMLChars($str) {
    return str_replace(array('&', '<', '>', '"'), array('&amp;', '&lt;', '&gt;', '&quot;'), $str);
  }
    

  /**
  * Checks a UTF-8 string for invalid bytes and converts it to UTF-8.
  *
  * It assumes that the invalid bytes are ISO-8859-1. Valid UTF-8 chars stay unchanged.
  *
  * @param string $str
  * @access public
  * @return string
  */
  function ensureUTF8($str) {
    $pattern = '~([\\xC2-\\xDF][\\x80-\\xBF]| #utf8-2
                  \\xE0[\\xA0-\\xBF][\\x80-\\xBF]|[\\xE1-\\xEC][\\x80-\\xBF]{2}| #utf8-3
                  \\xED[\\x80-\\x9F][\\x80-\\xBF]|[\\xEE-\\xEF][\\x80-\\xBF]{2}| #utf8-3
                  \\xF0[\\x90-\\xBF][\\x80-\\xBF]{2}|[\\xF1-\\xF3][\\x80-\\xBF]{3}| #utf8-4
                  \\xF4[\\x80-\\x8F][\\x80-\\xBF]{2}) #utf8-4
                  |([^\\x00-\\x7F])~x';
    return preg_replace_callback($pattern, 'ensureUTF8CharCallback', $str);
  }

  /**
  * Check the string to be valid UTF-8
  *
  * @param string $str
  * @access public
  * @return boolean String is valid UTF-8
  */
  function isUTF8($str) {    
    $pattern = '~^([\\x00-\\x7F]|[\\xC2-\\xDF][\\x80-\\xBF]|\\xE0[\\xA0-\\xBF][\\x80-\\xBF]|[\\xE1-\\xEC][\\x80-\\xBF]{2}|\\xED[\\x80-\\x9F][\\x80-\\xBF]|[\\xEE-\\xEF][\\x80-\\xBF]{2}|\\xF0[\\x90-\\xBF][\\x80-\\xBF]{2}|[\\xF1-\\xF3][\\x80-\\xBF]{3}|\\xF4[\\x80-\\x8F][\\x80-\\xBF]{2})*$~';    
    $result =  (bool)preg_match($pattern, $str);
    return $result;
  }

  /**
  * Check the string to contain only ASCII chars
  *
  * @param string $str
  * @access public
  * @return boolean String contains only ASCII chars
  */
  function isAscii($str) {
    $pattern = '~^([\\x00-\\x7F]+)$~';
    return (bool)preg_match($pattern, $str);
  }

  /**
  * Decodes a string using the constant PAPAYA_INPUT_ENCODING
  *
  * If iconv is not avaliable or the PAPAYA_INPUT_ENCODING no set, 
  * it converts unknown bytes from ISO-8859-1 to UTF-8
  *
  * @param string $str
  * @access public
  * @return string
  */
  function decodeInputData($str) {
    if (function_exists('iconv')) {
      if (defined('PAPAYA_INPUT_ENCODING') && strtoupper(PAPAYA_INPUT_ENCODING) != 'UTF-8') {
        return iconv(PAPAYA_INPUT_ENCODING, 'UTF-8', $str);
      } else {
        return papaya_strings::ensureUTF8($str);
      }
    } else {
      return papaya_strings::ensureUTF8($str);
    }
  }
  
  function substr($str, $start, $length = NULL) {
    if (function_exists('mb_substr')) {
      return mb_substr($str, $start, $length, 'UTF-8');
    } elseif (function_exists('iconv_substr')) {
      return iconv_substr($str, $start, $length, 'UTF-8');
    } else {
      return substr($str, $start, $length);
    }
  }
  
  function strlen($str) {
    if (function_exists('mb_strlen')) {
      return mb_strlen($str, 'UTF-8');
    } elseif (function_exists('iconv_strlen')) {
      return iconv_strlen($str, 'UTF-8');
    } else {
      return strlen($str);
    }
  }
  
  function normalizeString($utf8String, $maxLength = 0) {
    if (function_exists('iconv')) {
      $str = iconv('UTF-8', 'ISO-8859-1//TRANSLIT', $utf8String);
      $charReplace = array(
        "" => "A", "" => "A", "" => "A", "" => "A", "" => "A", "" => "a", "" => "a", "" => "a", "" => "a",
        "" => "a", "" => "C", "" => "c", "" => "E", "" => "E", "" => "E", "" => "E", "" => "e", "" => "e", 
        "" => "e", "" => "e", "" => "I", "" => "I", "" => "I", "" => "I", "" => "i", "" => "i", "" => "i",
        "" => "i", "" => "N", "" => "N", "" => "O", "" => "O", "" => "O", "" => "O", "" => "O", "" => "o",
        "" => "o", "" => "o", "" => "o", "" => "o", "" => "o", "" => "U", "" => "U", "" => "U", "" => "u",
        "" => "u", "" => "u", "" => "Y", "" => "y", "" => "y", 
        "" => "Ae","" => "Ue", "" => "Oe",
        "" => "ae", "" => "ue", "" => "oe", "" => "ss"
      );
      $str = strtr($str, $charReplace);      
    } else {
      $charReplace = array(
       'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Å' => 'A',
       'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a', 'å' => 'a', 'Ç' => 'C', 'ç' => 'c', 
       'È' => 'E', 'É' => 'E', 'Ê' => 'E', 'Ë' => 'E', 'è' => 'e', 'é' => 'e', 'ê' => 'e', 'ë' => 'e',    'Ì' => 'I', 'Í' => 'I', 'Î' => 'I', 'Ï' => 'I', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i', 
       'Ñ' => 'N', 'ñ' => 'N', 'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ø' => 'O', 
       'ð' => 'o', 'ò' => 'o', 'ó' => 'o', 'ô' => 'o', 'õ' => 'o', 'ø' => 'o', 'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U', 
       'ù' => 'u', 'ú' => 'u', 'û' => 'u', 'Ý' => 'Y', 'ý' => 'y', 'ÿ' => 'y', 'Ä' => 'Ae', 'Ü' => 'Ue', 'Ö' => 'Oe', 
       'ä' => 'ae', 'ü' => 'ue', 'ö' => 'oe', 'ß' => 'ss'
      );
      $str = strtr($utf8String, $charReplace);
    }
    $str = preg_replace('/[^a-zA-Z0-9-]/', '-', $str);
    $str = trim(preg_replace('/--+/', '-', $str), '-');
    if ($maxLength > 0 && strlen($str) > $maxLength) {
      $str = substr($str, 0, $maxLength);
      $p = strrpos($str, '-');
      if ($p > 0) {
        $str = substr($str, 0, $p);
      }
    }
    return $str;
  }
  
  function cleanInputString($value) {
    $special = array(
      '&nbsp;' => ' ',
      '&ndash;' => '-', 
      '&bdquo;' => '"', 
      '&ldquo;' => '"', 
      '&hellip;' => '...',
      '&lsquo;' => "'",
      '&rsquo;' => "'",
      '&euro;' => "€");
    $translation = array_flip(get_html_translation_table(HTML_ENTITIES, ENT_NOQUOTES));
    unset($translation['&gt;']);
    unset($translation['&lt;']);
    $result = strtr($value, array_merge($translation, $special));
    $remove = array(
      '~<(/?)o:\w*[^>]*>~i', 
      '~<(\w+)[^<>]*?[^/]></\\1>~i',
      '~<!--(.*?)-->~',
      '~<strong>\s*</strong>~iU',
      '~style="margin-bottom: 0cm;"~i'
    );
    $result = preg_replace($remove, '', $result);
    return papaya_strings::ensureUTF8($result);
  }
  
  function utf8ToCharset($utf8String, $charset) {
    if (function_exists('iconv')) {
      return iconv('UTF-8', strtoupper($charset).'//TRANSLIT', $utf8String);
    } elseif (function_exists('mb_convert_encoding')) {
      return mb_convert_encoding($utf8String, strtoupper($charset), 'UTF-8');
    }
    return utf8_decode($utf8String);
  }
}


/**
* Callback function for papaya_strings::ensureUTF8
*
* Can get a valid utf-8 sequence in $charMatch[1] or an invalid bytecode in $charMatch[2]
* If $charMatch[2] is filled papaya assumes that it is a ISO-8859-1 char, because old papaya CMS versions supported only that charset
*
* @param array $charMatch
* @access public
* @return string
*/
function ensureUTF8CharCallback($charMatch) {
  if (isset($charMatch[2]) && $charMatch[2] != '') {
    $c = ord($charMatch[2]);
    if ($c <= 8) {
      return '';
    }
    return chr(0xC0|$c>>6).chr(0x80|$c&0x3F);
  } else {
    return $charMatch[0];
  }
}

?>